import subprocess
import sys
import os
import psutil
from check_for_updates import check_updates

# Menu options as global variables
download = ""
download_install = ""
flatpak = ""
list_updates = ""
rofi_command = []
pid_file = "/tmp/update_menu.pid"

# Define translations
translations = {
    'en': {
        'download': "Download",
        'download_install': "Download and Install",
        'flatpak': "Flatpak",
        'reboot_install': "Reboot and Install",
        'install_next_reboot': "Install on Next Reboot",
        'do_nothing': "Do Nothing",
        'apply_updates': "Apply Updates",
        'update_success': "Update completed successfully",
        'error_occurred': "An error occurred",
        'update_in_progress': "Update is already in progress",
        'list_updates': "List Available Updates"
    },
    'pt': {
        'download': "Baixar",
        'download_install': "Baixar e Instalar",
        'flatpak': "Flatpak",
        'reboot_install': "Reiniciar e Instalar",
        'install_next_reboot': "Instalar no Próximo Reinício",
        'do_nothing': "Não Fazer Nada",
        'apply_updates': "Instalar Atualizações",
        'update_success': "Atualização concluída com sucesso",
        'error_occurred': "Ocorreu um erro",
        'update_in_progress': "Atualização já está em andamento",
        'list_updates': "Listar Atualizações Disponíveis"
    },
    'es': {
        'download': "Descargar",
        'download_install': "Descargar e Instalar",
        'flatpak': "Flatpak",
        'reboot_install': "Reiniciar e Instalar",
        'install_next_reboot': "Instalar en el Próximo Reinicio",
        'do_nothing': "No Hacer Nada",
        'apply_updates': "Instalar Atualizaciones",
        'update_success': "Actualización completada con éxito",
        'error_occurred': "Ocurrió un error",
        'update_in_progress': "La actualización ya está en progreso",
        'list_updates': "Listar Actualizaciones Disponibles"
    }
}

# Rofi theming
rofi_theme = '''
configuration{
    show-icons:false;
}
#window {
    width: 14em;
    anchor: north east;
    location: north east;
}
mainbox {
    children: [inputbar,listview];
}
listview{
    coluns:1;
    lines:4;
}
'''

def get_system_language():
    lang = os.environ.get('LANG', 'en_US.UTF-8')
    if lang.startswith('pt'):
        return 'pt'
    elif lang.startswith('es'):
        return 'es'
    else:
        return 'en'

def show_menu():
    menu = f"{download}\n{download_install}\n{flatpak}\n{list_updates}"
    result = subprocess.run(rofi_command, input=menu, text=True, capture_output=True)
    return result.stdout.strip()

def handle_post_download(lang):
    reboot = f"{translations[lang]['reboot_install']}"
    next_boot = f"{translations[lang]['install_next_reboot']}"
    do_nothing = f"{translations[lang]['do_nothing']}"
    menu = f"{reboot}\n{next_boot}\n{do_nothing}"
    result = subprocess.run(rofi_command, input=menu, text=True, capture_output=True)
    return result.stdout.strip()

def notify_send(message):
    subprocess.run(['notify-send', message])

def is_process_running(pid):
    return psutil.pid_exists(pid)

def handle_click(option, lang):
    try:
        if option == download and download != '':
            result = subprocess.run(['pkcon', '-p', '-n', '--only-download', 'update', '-y'], capture_output=True, text=True)
            notify_send(result.stdout.splitlines()[-1])
            post_option = handle_post_download(lang)
            handle_click(post_option, lang)
        elif option == download_install and download_install != '':
            result = subprocess.run(['pkcon', '-p', '-n', 'update', '-y'], capture_output=True, text=True)
            notify_send(result.stdout.splitlines()[-1])
        elif option == flatpak and flatpak != '':
            subprocess.run(['flatpak', 'update','-y', '--app'])
            notify_send(translations[lang]['update_success'])
        elif option == translations[lang]['reboot_install']:
            subprocess.run(['pkcon', 'offline-trigger'])
            subprocess.run(['reboot'])
        elif option == translations[lang]['install_next_reboot']:
            subprocess.run(['pkcon', 'offline-trigger'])
        elif option == list_updates and list_updates != '':
            pkcon_updates = subprocess.run(['pkcon', 'get-updates'], capture_output=True, text=True)
            flatpak_updates = subprocess.run(['flatpak', 'update', '--app', '--noninteractive'], capture_output=True, text=True)
            updates = pkcon_updates.stdout + "\n" + flatpak_updates.stdout
            notify_send(updates)
    except subprocess.CalledProcessError as e:
        notify_send(f"{translations[lang]['error_occurred']}: {e}")
    finally:
        if os.path.exists(pid_file):
            os.remove(pid_file)

def create_menu_options(lang):
    global download, download_install, flatpak, list_updates, rofi_command
    pkcon_update_count, flatpak_update_count = check_updates()
    download = f"{translations[lang]['download']} ({pkcon_update_count})"
    download_install = f"{translations[lang]['download_install']} ({pkcon_update_count})"
    flatpak = f"{translations[lang]['flatpak']} ({flatpak_update_count})"
    list_updates = translations[lang]['list_updates']
    rofi_command = ['rofi', '-dmenu', 'dmenu', '-theme-str', rofi_theme, '-i', '-p', translations[lang]['apply_updates']]

def main():
    lang = get_system_language()
    create_menu_options(lang)

    if os.path.exists(pid_file):
        with open(pid_file, 'r') as f:
            pid = int(f.read().strip())
            if is_process_running(pid):
                notify_send(translations[lang]['update_in_progress'])
                return

    with open(pid_file, 'w') as f:
        f.write(str(os.getpid()))

    try:
        if len(sys.argv) > 1:
            option = sys.argv[1]
            handle_click(option, lang)
        else:
            option = show_menu()
            handle_click(option, lang)
    finally:
        if os.path.exists(pid_file):
            os.remove(pid_file)

if __name__ == "__main__":
    main()
